'use strict';
/**
 流程模板配置和引擎调用接口的controller模块，实现了对外的URL接口，包括PC端和移动端

 注意点 :
 1. base.js继承自 think.Controller;
 2. 其他controller 继承自 base.js;
 3. 具体的业务模块可以继承并扩展 flow/model/act.js 来实现增加标准以外的逻辑
 4. 可以根据具体的业务模块选择适当基类，例如审核类:flow/model/act_appr.js
 5. 使用flow的页面统一从 controller/act.js, proc.js 提供的URL接口调用，也可继承并重写相应方法来增加逻辑（但一般从model/act.js, proc.js继承即可）

 @module flow.controller
 */

/**
 * 提供工作流模板的活动节点设置的URL接口<br/>
 * 提供工作流引擎的活动节点运转相关的URL调用接口
 * @class flow.controller.act
 */
const Base = require('./../base.js');
const moment = require('moment');

module.exports = class extends Base {

  /**
   * showdoc
   * @catalog 流程节点信息
   * @title 流程节点列表
   * @description 流程节点列表
   * @method post
   * @url /act/list
   * @param page 必选 int 第几页
   * @param limit 必选 int 每页多少条数据
   * @param procID 必选 int 主流程ID
   * @return {
   * "errno": 0,
   * "errmsg":"",
   * "data":{
   * "count": 1,
   *   "totalPages": 1,
   *   "pageSize": 1,
   *   "currentPage": 1,
   *   "data": [
   *       {
   *             "id": 18,
   *             "c_name": "开始", //名称
   *             "c_desc": "",  //描述
   *             "c_proc": 10,//所属模板ID
   *             "c_type": 3,//类型 1.人为参与 2.自动执行 3.开始节点 4.哑活动 9.结束节点
   *             "c_class": "flow/act", //实现类:
   *             "c_can_terminate": 0,//可以终止 0 不可以 1可以
   *             "c_can_suspend": 0,//可以挂起 0 不可以 1可以
   *             "c_call_btn": "",//按钮样式
   *             "c_form": "{modulename:'CustomerAppr'}",//表单链接
   *             "c_form_btn": "",//表单按钮
   *             "c_from_rule": 1,//前规则
   *             "c_votes": 0,//投票数
   *             "c_to_rule": 1,//后规则
   *             "c_cc_rule": 1,//通知规则
   *             "c_cc_title": "",//通知标题
   *             "c_cc_content": "",//通知内容
   *             "c_status": 0,//状态
   *             "c_domain_st": 1218,//业务状态
   *             "c_domain": "",//业务规则
   *             "c_memo": "",//备注
   *             "c_map_id": "rect1"//c_map_id 为图片中节点图形的ID
   *       }
   *   ]
   * }
   * }
   */
  async listAction() {
    const data = this.post();
    const procID = data.procID || 0;
    if (procID === 0) {
      this.fail();
      return;
    }
    const back = await this.model('fw_act').page(data.page, data.limit).order('id').where({
      c_status: 0, c_proc: procID
    }).countSelect();// 时间倒序
    this.success(back);
  }

  /**
   * showdoc
   * @catalog 流程节点信息
   * @title 流程节点增加
   * @description 流程节点增加
   * @method post
   * @url /act/add
   * @param c_name 必选 string 流程名称
   * @param c_desc 必选 string 流程描述
   * @param c_proc 必选 int 所属模板ID
   * @param c_type 非必选 int 类型 1.人为参与 2.自动执行 3.开始节点 4.哑活动 9.结束节点
   * @param c_class 非必选 string 实现类
   * @param c_desc 必选 string 流程描述
   * @param c_type 必选 int 流程类型 1.常规类型 2.审核类型 8.状态流转
   * @param c_can_terminate 必选 int 可以终止 0 不可以 1可以
   * @param c_can_suspend 必选 int 可以挂起 0 不可以 1可以
   * @param c_call_btn 非必选 string 按钮样式
   * @param c_form 非必选 string "{modulename:'CustomerAppr'}",//表单链接
   * @param c_form_btn 非必选 string 表单按钮
   * @param c_from_rule 非必选 string 前规则
   * @param c_votes 非必选 string 投票数
   * @param c_to_rule 非必选 string 后规则
   * @param c_cc_rule 非必选 string 通知规则
   * @param c_cc_title 非必选 string 通知标题
   * @param c_cc_content 必选 string 通知内容
   * @param c_status 必选 string 状态
   * @param c_domain_st 非必选 string 业务状态
   * @param c_domain 非必选 string 业务规则
   * @param c_memo 非必选 string 备注
   * @param c_map_id 非必选 string c_map_id 为图片中节点图形的ID
   * @return {
   * "errno": 0,
   * "errmsg":""
   * }
   * }
   */
  async addAction() {
    const addData = this.post();
    if (think.isEmpty(addData.id)) {
      const res = await this.model('fw_act').add(addData);
      if (res) {
        this.success({id: res});
      } else {
        this.fail();
      }
    }
  }

  /**
   * showdoc
   * @catalog 流程节点信息
   * @title 流程节点修改
   * @description 流程节点修改
   * @method post
   * @url /act/modify
   * @param id 必选 int id
   * @param c_name 必选 string 流程名称
   * @param c_desc 必选 string 流程描述
   * @param c_proc 必选 int 所属模板ID
   * @param c_type 非必选 int 类型 1.人为参与 2.自动执行 3.开始节点 4.哑活动 9.结束节点
   * @param c_class 非必选 string 实现类
   * @param c_desc 必选 string 流程描述
   * @param c_type 必选 int 流程类型 1.常规类型 2.审核类型 8.状态流转
   * @param c_can_terminate 必选 int 可以终止 0 不可以 1可以
   * @param c_can_suspend 必选 int 可以终止 0 不可以 1可以
   * @param c_call_btn 非必选 string 按钮样式
   * @param c_form 非必选 string "{modulename:'CustomerAppr'}",//表单链接
   * @param c_form_btn 非必选 string 表单按钮
   * @param c_from_rule 非必选 string 前规则
   * @param c_votes 非必选 string 投票数
   * @param c_to_rule 非必选 string 后规则
   * @param c_cc_rule 非必选 string 通知规则
   * @param c_cc_title 非必选 string 通知标题
   * @param c_cc_content 必选 string 通知内容
   * @param c_status 必选 string 状态
   * @param c_domain_st 非必选 string 业务状态
   * @param c_domain 非必选 string 业务规则
   * @param c_memo 非必选 string 备注
   * @param c_map_id 非必选 string c_map_id 为图片中节点图形的ID
   * @return {
   * "errno": 0,
   * "errmsg":""
   * }
   * }
   */
  async modifyAction() {
    const updateData = this.post();

    if (!think.isEmpty(updateData.id)) {
      const res = await this.model('fw_act').where({id: updateData.id}).update(updateData);
      if (res) {
        this.success();
      } else {
        this.fail();
      }
    }
  }

  /**
   * showdoc
   * @catalog 流程节点信息
   * @title 流程节点删除
   * @description 流程节点删除
   * @method post
   * @url /act/del
   * @param id 必选 int
   * @return {
   * "errno": 0,
   * "errmsg":""
   * }
   */
  async delAction() {
    const get = this.post();
    const id = get.id;
    const where = {id: id};
    const data = {c_status: -1};
    const res = await this.model('fw_act').where(where).update(data);
    if (res) {
      this.success();
    } else {
      this.fail();
    }
  }

  /**
   * showdoc
   * @catalog 流程节点信息
   * @title 流程路径列表
   * @description 流程路径列表
   * @method post
   * @url /act/pathlist
   * @param page 必选 int 第几页
   * @param limit 必选 int 每页多少条数据
   * @param procID 必选 int 主流程ID
   * @return {
   * "errno": 0,
   * "errmsg":"",
   * "data":{
   * "count": 1,
   *   "totalPages": 1,
   *   "pageSize": 1,
   *   "currentPage": 1,
   *   "data": [
   *       {
   *             "id": 18,
   *             "c_name": "开始", //名称
   *             "c_proc": 10,//所属模板ID
   *             "c_form": "",//从那个节点开始
   *             "c_to": "",//从哪个节点结束
   *             "c_status": 0,//状态
   *             "c_domain": "",//业务规则
   *             "c_memo": "",//备注
   *             "c_map_id": "rect1"//c_map_id
   *       }
   *   ]
   * }
   * }
   */
  async pathlistAction() {
    const data = this.post();
    const procID = data.procID || 0;
    if (procID === 0) {
      this.fail();
      return;
    }
    const back = await this.model('fw_act_path').page(data.page, data.limit).order('id').where({
      c_status: 0, c_proc: procID
    }).countSelect();// 时间倒序
    this.success(back);
  }
  /**
   * showdoc
   * @catalog 流程节点信息
   * @title 流程路径增加
   * @description 流程路径增加
   * @method post
   * @url /act/pathAdd
   * @param id 必选 int id
   * @param c_name 必选 string 流程名称
   * @param c_proc 必选 int 所属模板ID
   * @param c_form 必选 int 从那个节点开始
   * @param c_to 必选 int 从哪个节点结束
   * @param c_status 必选 int 状态 0启用 -1删除
   * @param c_domain 必选 string 业务规则
   * @param c_memo 必选 string 备注
   * @param c_map_id 必选 string c_map_id
   * @return {
   * "errno": 0,
   * "errmsg":""
   * }
   * }
   */
  async pathAddAction() {
    const addData = this.post();
    if (think.isEmpty(addData.id)) {
      const res = await this.model('fw_act_path').add(addData);
      if (res) {
        this.success({id: res});
      } else {
        this.fail();
      }
    }
  }

  /**
   * showdoc
   * @catalog 流程节点信息
   * @title 流程路径修改
   * @description 流程路径修改
   * @method post
   * @url /act/pathModify
   * @param id 必选 int id
   * @param c_name 必选 string 流程名称
   * @param c_proc 必选 int 所属模板ID
   * @param c_form 必选 int 从那个节点开始
   * @param c_to 必选 int 从哪个节点结束
   * @param c_status 必选 int 状态 0启用 -1删除
   * @param c_domain 必选 string 业务规则
   * @param c_memo 必选 string 备注
   * @param c_map_id 必选 string c_map_id
   * @return {
   * "errno": 0,
   * "errmsg":""
   * }
   * }
   */
  async pathModifyAction() {
    const updateData = this.post();
    if (!think.isEmpty(updateData.id)) {
      const res = await this.model('fw_act_path').where({id: updateData.id}).update(updateData);
      if (res) {
        this.success();
      } else {
        this.fail();
      }
    }
  }

  /**
   * showdoc
   * @catalog 流程节点信息
   * @title 流程路径删除
   * @description 流程路径删除
   * @method post
   * @url /act/pathDel
   * @param id 必选 int
   * @return {
   * "errno": 0,
   * "errmsg":""
   * }
   */
  async pathDelAction() {
    const get = this.post();
    const id = get.id;
    const where = {id: id};
    const data = {c_status: -1};
    const res = await this.model('fw_act_path').where(where).update(data);
    if (res) {
      this.success();
    } else {
      this.fail();
    }
  }

  //7.状态查询接口

  //流程初始化接口
  //流程查询下一个节点接口
  //流程查询上一个节点接口（包含跳转不同单位）
  //流程进入下一个节点接口（包含终止情况）
  //流程回退上一个节点接口

  /**
   * 编辑流程节点内容， GET调用： /flow/act/edit?id=xxx
   * @method  edit
   * @return {json}   act编辑页面的HTML片段
   */
  async editAction() {
    let actModule = cmpage.service('cmpage/module');
    let parmsAct = await actModule.getModuleByName('FwAct');
    parmsAct.parmsUrl = '{}';
    parmsAct.editID = this.get('id');
    parmsAct.user = await this.session('user');
    let pageAct = cmpage.service(parmsAct.c_path);
    pageAct.mod = parmsAct;
    await pageAct.initPage();
    pageAct.modEdits = await actModule.getModuleEdit(parmsAct.id);

    let actEditHtml = await pageAct.htmlGetEdit();

    return this.json({
      statusCode: 200,
      actEditHtml: actEditHtml
    });
  }

  /**
   * 删除流程节点并删除相关的路径信息， GET调用： /flow/act/delete?id=xxx
   * @method  delete
   * @return {json}   删除状态
   */
  async deleteAction() {
    let actID = this.get('id');
    await this.model('fw_act_path', 'cmpage').where(`c_from=${actID} or c_to=${actID}`).delete();
    await this.model('fw_act', 'cmpage').where({
      id: actID
    }).delete();

    return this.json({
      statusCode: 200,
      message: ''
    });
  }

  /**
   * 删除流程节点的路径信息， GET调用： /flow/act/delete_path?id=xxx
   * @method  deletePath
   * @return {json}   删除状态
   */
  async delete_pathAction() {
    let id = this.get('id');
    await this.model('fw_act_path', 'cmpage').where(`id=${id}`).delete();

    return this.json({
      statusCode: 200,
      message: ''
    });
  }

  /**
   * 编辑流程节点内容， GET调用： /flow/act/edit?id=xxx
   * @method  edit
   * @return {json}   act编辑页面的HTML片段
   */
  async get_act_pathAction() {
    let id = this.get('id');
    let data = await this.model('fw_act_path', 'cmpage').where({
      id: id
    }).find();

    return this.json({
      statusCode: 200,
      data: data
    });
  }

};
